@@ -1,5 +1,6 @@
-use fastnbt::de::from_bytes;
+#![allow(dead_code)]
 use fastnbt::error::Result;
+use fastnbt::from_bytes;
 use flate2::read::GzDecoder;
 use serde::Deserialize;
 use std::io::Read;
