@@ -2,11 +2,12 @@ use std::borrow::Cow;
 use std::collections::HashMap;
 
 use crate::error::{Error, Result};
-use crate::{de::from_bytes, Value};
+use crate::{from_bytes, Value};
 use crate::{ByteArray, IntArray, LongArray, Tag};
 
 use super::builder::Builder;
-use serde::Deserialize;
+use super::Single;
+use serde::{Deserialize, Serialize};
 
 #[test]
 fn error_impls_sync_send() {
@@ -14,6 +15,14 @@ fn error_impls_sync_send() {
     i(Error::invalid_tag(1));
 }
 
+#[test]
+fn descriptive_error_on_gzip_magic() {
+    let r = from_bytes::<()>(&[0x1f, 0x8b]);
+    assert!(matches!(r, Result::Err(_)));
+    let e = r.unwrap_err();
+    assert!(e.to_string().to_lowercase().contains("gzip"));
+}
+
 #[test]
 fn simple_byte() -> Result<()> {
     #[derive(Deserialize)]
@@ -560,14 +569,19 @@ fn optional() -> Result<()> {
 
 #[test]
 fn unit_just_requires_presense() -> Result<()> {
+    #[derive(Deserialize)]
+    struct Foo;
+
     #[derive(Deserialize)]
     struct V {
         _unit: (),
+        _unit_struct: Foo,
     }
 
     let payload = Builder::new()
         .start_compound("object")
         .byte("_unit", 0)
+        .byte("_unit_struct", 0)
         .end_compound()
         .build();
 
@@ -824,7 +838,7 @@ fn vec_from_nbt_byte_array() -> Result<()> {
         .build();
 
     let v: V = from_bytes(payload.as_slice())?;
-    assert_eq!(*v.a, [1, 2, 3]);
+    assert!(v.a.iter().eq(&[1, 2, 3]));
     assert_eq!(*v.b, [4, 5, 6]);
     assert_eq!(*v.c, [7, 8, 9]);
     Ok(())
@@ -922,12 +936,12 @@ fn fixed_array() -> Result<()> {
 fn type_mismatch_string() -> Result<()> {
     #[derive(Deserialize, Debug)]
     pub struct V {
-        a: String,
+        _a: String,
     }
 
     let payload = Builder::new()
         .start_compound("object")
-        .int("a", 123)
+        .int("_a", 123)
         .end_compound() // end of outer compound
         .build();
 
@@ -943,6 +957,7 @@ fn basic_palette_item() -> Result<()> {
     #[serde(rename_all = "PascalCase")]
     pub struct PaletteItem {
         name: String,
+        #[allow(dead_code)]
         properties: HashMap<String, String>,
     }
 
@@ -1109,7 +1124,7 @@ fn byte_array_in_fullvalue() {
         Value::Compound(ref map) => {
             let a = &map["a"];
             match a {
-                Value::ByteArray(arr) => assert_eq!(&**arr, &[1, 2, 3]),
+                Value::ByteArray(arr) => assert!(arr.iter().eq(&[1, 2, 3])),
                 _ => panic!("{:?}", a),
             }
         }
@@ -1170,7 +1185,7 @@ fn cesu8_string_in_nbt() {
 fn cannot_borrow_cesu8_if_diff_repr() {
     #[derive(Deserialize, Debug)]
     pub struct V<'a> {
-        name: &'a str,
+        _name: &'a str,
     }
 
     let modified_unicode_str = cesu8::to_java_cesu8("😈");
@@ -1178,7 +1193,7 @@ fn cannot_borrow_cesu8_if_diff_repr() {
     let input = Builder::new()
         .start_compound("")
         .tag(Tag::String)
-        .name("name")
+        .name("_name")
         .raw_len(modified_unicode_str.len())
         .raw_bytes(&modified_unicode_str)
         .end_compound()
@@ -1239,3 +1254,105 @@ fn can_cow_cesu8() {
     assert!(matches!(v.borrowed, Cow::Borrowed(_)));
     assert_eq!("abc", v.borrowed);
 }
+
+#[test]
+fn large_list() {
+    let input = [10, 0, 0, 9, 0, 0, 10, 4, 0, 5, 252];
+    let _v: Result<Value> = from_bytes(&input);
+}
+
+#[test]
+fn hashmap_with_bytes() {
+    // Users should be able to decode strings as borrowed byte strings if they
+    // really want to.
+    let input = Builder::new()
+        .start_compound("")
+        .string("hello", "😈")
+        .end_compound()
+        .build();
+
+    let v: HashMap<&[u8], &[u8]> = from_bytes(&input).unwrap();
+    assert_eq!(
+        cesu8::from_java_cesu8(v["hello".as_bytes()])
+            .unwrap()
+            .as_bytes(),
+        "😈".as_bytes()
+    );
+}
+
+#[test]
+fn hashmap_with_byte_buf() {
+    // Users should be able to decode strings as borrowed byte strings if they
+    // really want to.
+    let input = Builder::new()
+        .start_compound("")
+        .string("hello", "😈")
+        .end_compound()
+        .build();
+
+    let _v: HashMap<&[u8], serde_bytes::ByteBuf> = from_bytes(&input).unwrap();
+}
+
+#[test]
+fn chars() {
+    let input = Builder::new()
+        .start_compound("")
+        .string("val", "a")
+        .end_compound()
+        .build();
+    let v: Single<char> = from_bytes(&input).unwrap();
+    assert_eq!('a', v.val);
+}
+
+#[test]
+fn enum_variant_types() {
+    #[derive(Serialize, Deserialize, Debug, PartialEq)]
+    enum Letter {
+        NewType(u32),
+        Tuple(u8, u8, u8),
+        Struct { a: String },
+    }
+
+    let newtype_input = Builder::new()
+        .start_compound("")
+        .string("val", "NewType")
+        .end_compound()
+        .build();
+    let v: Result<Single<Letter>> = from_bytes(&newtype_input);
+    assert!(matches!(v, Err(_)));
+
+    let tuple_input = Builder::new()
+        .start_compound("")
+        .string("val", "Tuple")
+        .end_compound()
+        .build();
+
+    let v: Result<Single<Letter>> = from_bytes(&tuple_input);
+    assert!(matches!(v, Err(_)));
+
+    let struct_input = Builder::new()
+        .start_compound("")
+        .string("val", "Struct")
+        .end_compound()
+        .build();
+    let v: Result<Single<Letter>> = from_bytes(&struct_input);
+    assert!(matches!(v, Err(_)));
+}
+
+#[test]
+fn tuple_struct() {
+    #[derive(Deserialize, Serialize)]
+    struct Rgb(u8, u8, u8);
+
+    let input = Builder::new()
+        .start_compound("")
+        .start_list("val", Tag::Byte, 3)
+        .byte_payload(1)
+        .byte_payload(2)
+        .byte_payload(3)
+        .end_compound()
+        .build();
+
+    let v: Single<Rgb> = from_bytes(&input).unwrap();
+    assert!(matches!(v.val, Rgb(1, 2, 3)));
+}
